@@ -392,10 +392,11 @@ class LiveDataService(DataServiceBase):
 
     def upload_batch(self):
         # get most recent upload on queue
-        while self.queue:
+        upload = self.queue.popleft()
+        while self.queue and self.queue[0] is not None:
             upload = self.queue.popleft()
-            if upload is None:
-                return False
+        if upload is None:
+            return False
         timestamp, prepared_data = upload
         # check time since last upload
         if timestamp and timestamp < self.last_update + self.interval:
@@ -442,6 +443,10 @@ class FileService(ServiceBase):
                     path = upload
                 else:
                     path = os.path.join(self.context.output_dir, upload)
+                if not os.path.isfile(path):
+                    if upload in pending:
+                        pending.remove(upload)
+                    continue
                 self.logger.debug('file: %s', path)
                 OK, message = self.upload_file(session, path)
                 self.log(message)
